/***************************************************************************
 *   Copyright (C) 2006 by Vladimir Jary
 *   vljary.urk@centrum.cz
 *                                                                   
 *   This program is free software; you can redistribute it and/or modify  
 *   it under the terms of the GNU General Public License as published by 
 *   the Free Software Foundation; either version 2 of the License, or     
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful, 
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the 
 *   Free Software Foundation, Inc., 
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. 
 ***************************************************************************/


void mainForm::init()
{
	QIntValidator *validatorVQ = new QIntValidator(1,16000,this);
	QIntValidator *validatorAQ = new QIntValidator(32,500,this);
	QIntValidator *validator = new QIntValidator(this);
	validator->setBottom(0);
	editVideoRate->setValidator(validatorVQ);
	editAudioRate->setValidator(validatorAQ);
	editX->setValidator(validator);
	editY->setValidator(validator);
	editHz->setValidator(validator);
	editTop->setValidator(validator);
	editBottom->setValidator(validator);
	editLeft->setValidator(validator);
	editRight->setValidator(validator);
	process = 0;
	help = 0;
	parsed = false;
	editOutput->setText(QDir::homeDirPath()+"/");
	timeStart = 0;
}

void mainForm::showVideoForm()
{
	videoForm vf;
	connect(&vf,SIGNAL(inserted(QStringList)),this,SLOT(insertVideo(QStringList)));
	vf.setModal(true);
	vf.exec();
}


void mainForm::insertVideo( QStringList tags )
{
	QListViewItem *item = new QListViewItem(viewSource);
	
	int i = 0;
	for ( QStringList::Iterator it = tags.begin(); it != tags.end(); ++it ) {
		
		item->setText(i,*it);
		i++;
	}
}


void mainForm::selectOutput()
{
	QString directory = QFileDialog::getExistingDirectory(
			QDir::homeDirPath(),
			this,
			"get existing directory",
			tr("Choose an output directory"),
			TRUE );
	if(!directory.isEmpty())
	{
		editOutput->setText(directory);
	}
}

void mainForm::startProcess()
{
	QListViewItem *i = viewSource->firstChild();
	if (i!=0) {
		process = new QProcess(this);	
		process->addArgument("ffmpeg2theora");	
		process->addArgument(i->text(0)); //INPUT file
		lblInput->setText(tr("Input file: %1").arg(i->text(0)));
		QFileInfo fi(i->text(0));
		QString ext = "." + fi.extension();
		QString outputFile = i->text(0).replace(ext,".ogg");
		process->addArgument("--output");
		
		if((radioDestination->isOn()) && (!editOutput->text().isEmpty()))
		{
			fi.setFile(outputFile);
			outputFile = editOutput->text()  + fi.fileName();
		}
		process->addArgument(outputFile);
		lblOutput->setText(tr("Output file: %1").arg(outputFile));
		if(!comboAudio->currentText().isEmpty())
		{
			process->addArgument("--audioquality");
			process->addArgument(comboAudio->currentText());
		}
		
		if(!(editAudioRate->text().isEmpty()))
		{
			process->addArgument("-A");
			process->addArgument(editAudioRate->text());
		}
		
		if(!(editHz->text().isEmpty()))
		{
			process->addArgument("--samplerate");
			process->addArgument(editHz->text());
		}	
		
		if(!(editChannels->text().isEmpty()))
		{
			process->addArgument("--channels");
			process->addArgument(editChannels->text());
		}
		
		if(checkSound->isOn())
		{
			process->addArgument("--nosound");
		}
		
		if(!comboVideo->currentText().isEmpty())
		{
			process->addArgument("--videoquality");
			process->addArgument(comboVideo->currentText());
		}
		
		if(!(editVideoRate->text().isEmpty()))
		{
			process->addArgument("-V");
			process->addArgument(editVideoRate->text());
		}
		
		//Crop
		if(!(editTop->text().isEmpty()))
		{
			process->addArgument("--croptop");
			process->addArgument(editTop->text());
		}
		
		if(!(editBottom->text().isEmpty()))
		{
			process->addArgument("--cropbottom");
			process->addArgument(editBottom->text());
		}
		
		if(!(editLeft->text().isEmpty()))
		{
			process->addArgument("--cropleft");
			process->addArgument(editLeft->text());
		}
		
		if(!(editRight->text().isEmpty()))
		{
			process->addArgument("--cropright");
			process->addArgument(editRight->text());
		}
		
		if(!(editX->text().isEmpty()))
		{
			process->addArgument("--width");
			process->addArgument(editX->text());
		}
		
		if(!(editY->text().isEmpty()))
		{
			process->addArgument("--height");
			process->addArgument(editY->text());
		}
		
		if(!(editAspect->text().isEmpty()))
		{
			process->addArgument("--aspect");
			process->addArgument(editAspect->text());
		}
		
		if(!(editFPS->text().isEmpty()))
		{
			process->addArgument("--inputFPS");
			process->addArgument(editFPS->text());
		}
		
		if(!(i->text(1).isEmpty()))
		{
			process->addArgument("--artist");
			process->addArgument(i->text(1));
		}
		
		if(!(i->text(2).isEmpty()))
		{
			process->addArgument("--title");
			process->addArgument(i->text(2));
		}
		
		if(!(i->text(3).isEmpty()))
		{
			process->addArgument("--date");
			process->addArgument(i->text(3));
		}
		
		if(!(i->text(4).isEmpty()))
		{
			process->addArgument("--location");
			process->addArgument(i->text(4));
		}
		
		if(!(i->text(5).isEmpty()))
		{
			process->addArgument("--organization");
			process->addArgument(i->text(5));
		}
		
		if(!(i->text(6).isEmpty()))
		{
			process->addArgument("--copyright");
			process->addArgument(i->text(6));
		}
		
		if(!(i->text(7).isEmpty()))
		{
			process->addArgument("--licence");
			process->addArgument(i->text(7));
		}
		
		//Advanced options
		if(checkAdvanced->isOn()){
			if(!editStart->text().isEmpty())
			{
				process->addArgument("-s");
				process->addArgument(editStart->text());
			}
			
			if(!editEnd->text().isEmpty())
			{
				process->addArgument("-e");
				process->addArgument(editEnd->text());
			}
			if(spinBoxKeyframe->value()!=64)
			{
				process->addArgument("-K");
				QString kf = QString::number(spinBoxKeyframe->value());
				process->addArgument(kf);
			}
			if(sliderSharp->value()!=20)
			{
				QString sharp = QString::number(sliderSharp->value()/10);
				process->addArgument("-S");
				process->addArgument(sharp);
			}
			
			if(checkDeinterlace->isOn())
			{
				process->addArgument("--deinterlace");
			}
			
			if(checkOptimize->isOn())
			{
				process->addArgument("--optimize");
			}
			
			if(checkSync->isOn())
			{
				process->addArgument("--sync");
			}
			
			if(!editInFormat->text().isEmpty())
			{
				process->addArgument("--format");
				process->addArgument(editInFormat->text());
			}
			
			if(!editAudioStream->text().isEmpty())
			{
				process->addArgument("--audiostream");
				process->addArgument(editAudioStream->text());
			}
			
			if(!editNice->text().isEmpty())
			{
				process->addArgument("--nice");
				process->addArgument(editNice->text());
			}
			
			if(!editV4L->text().isEmpty())
			{
				process->addArgument("--v4l");
				process->addArgument(editV4L->text());
			}	
		}
		
		delete i;
		connect(process,SIGNAL(readyReadStderr()),this,SLOT(updateLog()));
		connect(process,SIGNAL(processExited()),this,SLOT(completed()));
		process->start();
	}
}

void mainForm::start()
{
	log->clear();
	if(viewSource->childCount()==0)
	{
		log->append(tr("Task queue is empty!"));
		return;
	}
	aborted = false;
	btnStart->setEnabled(false);
	btnStop->setEnabled(true);
	setFinishEnabled(pageConversion,false);
	header = QString::null;
	log->append(tr("Starting ..."));
	startProcess();
}

void mainForm::updateLog()
{
	QString text = (QString)(process->readStderr());
	text = text.simplifyWhiteSpace();
	text = text.stripWhiteSpace();
	
	QDateTime d(QDateTime::currentDateTime());
	if(!timeStart)
		timeStart = d.toTime_t();
	uint elapsed = d.toTime_t()-timeStart;
	uint hr = (uint)(elapsed/3600);
	uint min = (uint)((elapsed-3600*hr)/60);
	uint sec = (uint)(elapsed - 60*min-hr*3600);
	QTime elapsedTime(hr,min,sec);
	if(progressCurrent->progress()>0)
	{
		float remaining = (progressCurrent->totalSteps()*elapsed/progressCurrent->progress())-elapsed;
		hr = (uint)(remaining/3600);
		min = (uint)((remaining - 3600*hr)/60);
		sec = (uint)(remaining - 60*min-hr*3600);
		QTime remainingTime(hr, min, sec);
		// lblRemaining->setText(tr("%1:%2:%3").arg(hr).arg(min).arg(sec));
		lblRemaining->setText(tr("Remaing time: %1").arg(remainingTime.toString("hh:mm:ss")));
	}
	else
		lblRemaining->setText(tr("Remaing time: ??:??"));
	lblElapsed->setText(tr("Elapsed time: %1").arg(elapsedTime.toString("hh:mm:ss")));
	
	if(!parsed)
	{
		header += text;
		if(text.lower().contains("bitrate"))
		{
			log->append(header);
			QStringList sl = QStringList::split(":",header);
			//if(sl.count()>=5)
			//{
			QString dur = sl[2]+":"+sl[3]+":"+sl[4].left(2);
			lblLength->setText(tr("Length %1").arg(dur));
			int duration = sl[2].toInt()*3600 + sl[3].toInt()*60 + sl[4].left(2).toInt();
			progressCurrent->setTotalSteps(duration);
			parsed = true;
			//}
		} 
	} else
	{	
		if(text[0].isDigit())
		{
			QStringList sl = QStringList::split(":",text);
			if(sl.count()>=3)
			{
				QString pos = sl[0]+":"+sl[1]+":"+sl[2].left(2);
				lblPosition->setText(tr("Position: %1").arg(pos));
				int current = sl[0].toInt()*3600 + sl[1].toInt()*60 + sl[2].left(2).toInt();
				if(parsed)
					progressCurrent->setProgress(current);
			}
		}
		else //if(parsed)
			log->append(text);
	}	
}

void mainForm::completed()
{
	timeStart = 0;
	if(process->normalExit())
	{
		log->append(tr("Conversion completed with success!"));
	} else
	{
		log->append(tr("Conversion failed!"));
	}
	delete process;
	process = 0;
	parsed = false;
	if(viewSource->childCount()>0)
		startProcess();
	else
	{
		btnStart->setEnabled(true);
		btnStop->setEnabled(false);
		setFinishEnabled(pageConversion,true);
	}
}

void mainForm::killProcess()
{
	aborted = true;
	parsed = false;
	if(process)
	{	
		process->tryTerminate();
		QTimer::singleShot( 5000, process, SLOT( kill() ) );
	}
	log->append(tr("Aborted by user"));
}

void mainForm::reject()
{
	if(btnStop->isEnabled()){
		QMessageBox::information(this,tr("ui4theora"),tr("Conversion is in progress. Abort it if you wish to close this application"));
	}
	else
		QWizard::reject();
}

void mainForm::remove()
{
	QListViewItem *i = viewSource->currentItem();
	if(i)
	{
		viewSource->takeItem(i);
		delete i;
	}
	
}


void mainForm::saveConfig()
{
	QString fn = QFileDialog::getSaveFileName(
			QDir::homeDirPath(),  tr("Config files (*)"),
			this,  "save file dialog",  tr("Choose a filename to save under"));
	if(!fn.isEmpty())
	{
		QFile cfg_file(fn);
		if(cfg_file.exists())
			if(QMessageBox::question(this,tr("ui4theora"),tr("Selected file already exists.\nDo you wish to overwrite it?"),tr("&Yes"),tr("&No")))
				return;
		
		
		if(!cfg_file.open(IO_WriteOnly)){
			QMessageBox::information( this, tr("ui4theora"),
																tr("Cannot save the file.\n"
																	 "Make sure you have writing rights."));
			return;
		}
		
		QTextStream ts(&cfg_file);
		ts.setEncoding(QTextStream::UnicodeUTF8);
		ts << tr("# This file is genereted by ui4theora. Do not edit manualy.") << endl;
		ts << tr("# Audio setup") << endl;
		ts << "audioquality=" << comboAudio->currentText() << endl;
		ts << "audiobitrate=" << editAudioRate->text() << endl;
		ts << "samplerate=" << editHz->text() << endl;
		ts << "channels=" << editChannels->text() << endl;
		if(checkSound->isOn())
			ts << "sound=off" << endl;
		else
			ts << "sound=on" <<endl;
		ts << "# Video options" << endl;
		ts << "videoquality=" << comboVideo->currentText() << endl;
		ts << "videobitrate=" << editVideoRate->text() << endl;
		ts << "width=" << editX->text() << endl;
		ts << "height=" << editY->text() << endl;
		ts << "inputfps=" << editFPS->text() << endl;
		ts << "aspect=" << editAspect->text() << endl;
		ts << "croptop=" << editTop->text() << endl;
		ts << "cropbottom=" << editBottom->text() << endl;
		ts << "cropleft=" << editLeft->text() << endl;
		ts << "cropright=" << editRight->text() << endl;
		ts << "#[EOF]#" << endl;
	}
}

void mainForm::loadConfig()
{
	QString fn = QFileDialog::getOpenFileName(
			QDir::homeDirPath(),  tr("Config files (*)"),
			this,  "open file dialog",  tr("Select desired configuration file"));
	if(fn.isEmpty())
		return;
	QFile cfg_file(fn);
	if(!cfg_file.open(IO_ReadOnly))
	{
		QMessageBox::information( this, tr("ui4theora"),
															tr("Cannot open the file.\n"
																 "Permission denied."));
		return;
	}
	
	
	QTextStream ts(&cfg_file);
	ts.setEncoding(QTextStream::UnicodeUTF8);
	while(!ts.atEnd())
	{
		QString line = ts.readLine();
		line = line.simplifyWhiteSpace();
		line = line.lower();
		if(!(line.left(1)=="#")){
			
			
			QString argument = line.section('=',0,0);
			QString value = line.section('=',1,1);
			//Audio options
			if(argument == "audioquality")
				comboAudio->setCurrentText(value);
			else if(argument == "audiobitrate")
				editAudioRate->setText(value);
			else if(argument == "samplerate")
				editHz->setText(value);
			else if(argument == "channels")
				editChannels->setText(value);
			else if(argument == "sound")
			{
				if(value == "on")
					checkSound->setChecked(false);
				else
					checkSound->setChecked(true);
			}
			//Video options
			else if(argument == "videoquality")
				comboVideo->setCurrentText(value);
			else if(argument == "videobitrate")
				editVideoRate->setText(value);
			else if(argument == "width")
				editX->setText(value);
			else if(argument == "height")
				editY->setText(value);
			else if(argument == "inputfps")
				editFPS->setText(value);
			else if(argument == "aspect")
				editAspect->setText(value);
			//Crop
			else if(argument == "croptop")
				editTop->setText(value);
			else if(argument == "cropright")
				editRight->setText(value);
			else if(argument == "cropleft")
				editLeft->setText(value);
			else if(argument == "cropbottom")
				editBottom->setText(value);
		}
	}
}

void mainForm::resetConfig()
{
	if(QMessageBox::question(this,tr("ui4theora"),tr("Are you sure you wish to load default settings?"),tr("&Yes"),tr("&No")))
		return;
	comboAudio->setCurrentText("");
	checkSound->setChecked(false);
	editAudioRate->setText("");
	editHz->setText("");
	editChannels->setText("");
	
	comboVideo->setCurrentText("");
	editVideoRate->setText("");
	editFPS->setText("");
	editX->setText("");
	editY->setText("");
	editAspect->setText("");
	
	editTop->setText("");
	editBottom->setText("");
	editLeft->setText("");
	editRight->setText("");
}


void mainForm::showHelp()
{
	if(help)
		return;
   help = new QProcess(this);	
	help->addArgument("konqueror");	
	help->addArgument("man:/ffmpeg2theora");
	connect(help,SIGNAL(processExited()),this,SLOT(helpClosed()));
	help->start();
}

void mainForm::helpClosed()
{
	delete help;
	help = 0;
}


void mainForm::aboutQT()
{
	QMessageBox::aboutQt(this,tr("About QT"));
}


void mainForm::about()
{
	QMessageBox::about(this, tr("About ffmpeg2theora_fe"),tr("<h2>ffmpeg2theora_fe: Release 1</h2>"
								 
								"<p>ffmpeg2theora_fe is graphical user interface for <i>ffmpeg2theora</i></p>"
								"<p>created by Vladimir Jary</p>"
								"<p>E-mail: <a href=\"mailto:vljary.urk@centrum.cz\">vljary.urk@centrum.cz</a></p>"
								"<p>Web: <a href=\"http://kmlinux.fjfi.cvut.cz/~jaryv1\">http://kmlinux.fjfi.cvut.cz/~jaryv1</a></p>"
								"<p>This program may be used, modified and distributed under terms of  GPL</p>"));
}
